'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _socket = require('socket.io');

var _socket2 = _interopRequireDefault(_socket);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new _bluebird2.default(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return _bluebird2.default.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

class BeltlineServer {
  constructor(app, databaseConnection) {
    this.db = databaseConnection;

    this.io = (0, _socket2.default)(app);

    this.publishHandlers = {};
    this.queries = {};
    this.methods = {};
    this.isServer = true;

    this.io.on('connection', socket => {
      socket.on('subscribe', ({ id, subscriptionName, params }) => this.onSubscribe(id, subscriptionName, params, socket));
      socket.on('call', ({ methodName, params, callId }) => this.onCall(methodName, params, callId, socket));
      socket.on('unsubscribe', ({ id }) => this.onUnsubscribe(id, socket));
      socket.on('disconnect', () => this.onDisconnect(socket));
    });
  }

  onSubscribe(id, subscriptionName, params, socket) {
    var _this = this;

    return _asyncToGenerator(function* () {
      if (_this.publishHandlers[subscriptionName]) {
        const query = _this.publishHandlers[subscriptionName](params);
        if (_this.queries[query]) {
          _this.queries[query].push({ id, subscriptionName, socket });
        } else {
          _this.queries[query] = [{ id, subscriptionName, socket }];
        }
        const graph = yield _this.db.execute(query);
        socket.emit('published', {
          id,
          graph: graph.toNT(),
          query
        });
      }
    })();
  }

  onCall(methodName, params, callId) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      yield _this2.methods[methodName](params, _this2.db);
      yield _bluebird2.default.map(Object.keys(_this2.queries), (() => {
        var _ref = _asyncToGenerator(function* (query) {
          const clientInfo = _this2.queries[query];
          const graph = yield _this2.db.execute(query);
          clientInfo.forEach(function (client) {
            client.socket.emit('update', {
              id: client.id,
              graph: graph.toNT(),
              callId
            });
          });
        });

        return function (_x) {
          return _ref.apply(this, arguments);
        };
      })());
    })();
  }

  onUnsubscribe(id) {
    Object.keys(this.queries).forEach(query => {
      this.queries[query] = this.queries[query].filter(clientInfo => clientInfo.id !== id);
    });
  }

  onDisconnect(socket) {
    Object.keys(this.queries).forEach(query => {
      this.queries[query] = this.queries[query].filter(clientInfo => clientInfo.socket !== socket);
    });
  }

  publish(subscriptionName, publishHandler) {
    this.publishHandlers[subscriptionName] = publishHandler;
  }

  method(methodName, method) {
    this.methods[methodName] = method;
  }
}
exports.default = BeltlineServer;